import 'dart:typed_data';

import 'package:binarize/src/payload_type.dart';
import 'package:test/test.dart';

class TypeTester<T> {
  final PayloadType<T> type;

  final Uint8List data;

  ByteData get byteData {
    return ByteData.view(Uint8List.fromList(data).buffer);
  }

  final ByteData setData;

  final T value;

  final int byteLength;

  TypeTester({
    required this.type,
    required this.value,
    required List<int> data,
    required this.byteLength,
  })  : data = Uint8List.fromList(data),
        setData = ByteData.view(
          Uint8List.fromList(List.filled(type.length(value), 0)).buffer,
        );
}

void main() {
  test('Should test the type when given', () {
    expect(string.isType('value'), true);
    expect(string.isType(1), false);
  });

  final tests = [
    // Boolean tests
    TypeTester(type: boolean, value: false, data: [0], byteLength: 1),
    TypeTester(type: boolean, value: true, data: [1], byteLength: 1),

    // Flag tests
    TypeTester(
      type: flags,
      value: [true, false, true, false, false, false, false, false],
      data: [5],
      byteLength: 1,
    ),
    TypeTester(
      type: flags,
      value: [false, false, false, false, false, false, false, true],
      data: [128],
      byteLength: 1,
    ),

    // Float32 tests
    TypeTester(type: float32, value: 1.5, data: [63, 192, 0, 0], byteLength: 4),

    // Float64 tests
    TypeTester(
      type: float64,
      value: 1.5,
      data: [63, 248, 0, 0, 0, 0, 0, 0],
      byteLength: 8,
    ),

    // Int8 tests
    TypeTester(type: int8, value: -8, data: [248], byteLength: 1),
    TypeTester(type: int8, value: 8, data: [8], byteLength: 1),

    // Int16 tests
    TypeTester(type: int16, value: -16, data: [255, 240], byteLength: 2),
    TypeTester(type: int16, value: 16, data: [0, 16], byteLength: 2),

    // Int32 tests
    TypeTester(
      type: int32,
      value: -32,
      data: [255, 255, 255, 224],
      byteLength: 4,
    ),
    TypeTester(type: int32, value: 32, data: [0, 0, 0, 32], byteLength: 4),

    // Int64 tests
    TypeTester(
      type: int64,
      value: -64,
      data: [255, 255, 255, 255, 255, 255, 255, 192],
      byteLength: 8,
    ),
    TypeTester(
      type: int64,
      value: 64,
      data: [0, 0, 0, 0, 0, 0, 0, 64],
      byteLength: 8,
    ),

    TypeTester(
      type: string,
      value: 'Hello World',
      data: [0, 0, 0, 11, 72, 101, 108, 108, 111, 32, 87, 111, 114, 108, 100],
      byteLength: 4 + 'Hello World'.length,
    ),
    TypeTester(
      type: string,
      value: '',
      data: [0, 0, 0, 0],
      byteLength: 4 + ''.length,
    ),

    // Uint8 tests
    TypeTester(type: uint8, value: 100, data: [100], byteLength: 1),

    // Uint16 tests
    TypeTester(type: uint16, value: 10000, data: [39, 16], byteLength: 2),

    // Uint32 tests
    TypeTester(
      type: uint32,
      value: 1000000,
      data: [0, 15, 66, 64],
      byteLength: 4,
    ),

    // Uint64 tests
    TypeTester(
      type: uint64,
      value: 10000000000,
      data: [0, 0, 0, 2, 84, 11, 228, 0],
      byteLength: 8,
    ),
  ];

  for (final typeTest in tests) {
    test(
      '${typeTest.type.runtimeType} should return ${typeTest.value}',
      () => expect(typeTest.type.get(typeTest.byteData, 0), typeTest.value),
    );
    test(
      '${typeTest.type.runtimeType} should set bytes to the value of ${typeTest.value}',
      () {
        typeTest.type.set(typeTest.value, typeTest.setData, 0);
        expect(typeTest.setData.buffer.asUint8List(), typeTest.data);
      },
    );
    test(
      '${typeTest.type.runtimeType} should return a byte length of ${typeTest.byteLength}',
      () => expect(typeTest.type.length(typeTest.value), typeTest.byteLength),
    );
  }
}
